<?php

namespace Controller;

use Auth\AuthException;
use Event\TemplateEvent;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Template;
use Application;
use Client;
use Pdf;
use Boutique;

/**
 * Class Controller
 *
 * @package Controller
 */
class Controller
{

    /**
     * @var Request
     */
    protected $request;

    /**
     * @var \SiteSmarty
     */
    protected $template;

    /**
     * @var \DebugBar\DediDebugBar
     */
    protected $debugBar;

    /**
     * Constructor
     *
     * @param Request $request the current request
     */
    public function __construct(Request $request)
    {
        $this->request = $request;
        $this->template = Template::create('smarty');
        $this->debugBar = Application::getDebugBar();
    }

    /**
     * assign variable to use in the template
     *
     * @param array $data variable to assign
     *
     * @return void
     */
    public function assign($data)
    {
        foreach ($data as $key => $d) {
            $this->template->assign($key, $d);
        }
    }

    /**
     * return instance of controller give in parameter
     *
     * @param $controllerName
     *
     * @return Controller
     *
     * @throws \DediException
     */
    public function getController($controllerName)
    {
        $controllerObject = new \ReflectionClass($controllerName);

        if ($controllerObject->isSubclassOf('Controller\\Controller') == false) {
            throw new \DediException('the class ' . $controllerName . ' must extend Controller class');
        }

        return new $controllerName($this->request);
    }

    /**
     * Permet de généré une url
     *
     * @param string $name          the name of the url
     * @param array  $parameters    array of parameters for generate the url
     * @param string $referenceType The type of reference to be generated
     *
     * @return string
     */
    public function generateUrl($name, $parameters = array(), $referenceType = null)
    {
        try {
            return Application::getRouter()
                ->getRouter()
                ->generate($name, $parameters, $referenceType);
        } catch (\Exception $e) {
            if (!empty($this->debugBar)) {
                $this->debugBar['exceptions']->addException($e);
            }

            return false;
        }
    }

    /**
     * Retourne le client loggé ou false si l'internaute n'est pas loggé
     *
     * @return bool|Client
     */
    public function getCurrentClient()
    {
        $session = $this->request->getSession();

        if ($session->has('client_id')) {
            return new Client($session->get('client_id'));
        } else {
            return false;
        }
    }

    /**
     * Permet de verifier un role
     *
     * @param string $roleName the name of the role
     *
     * @return mixed
     *
     * @throws AuthException
     * @throws \DediException
     */
    public function checkRole($roleName)
    {
        $result = Application::getAuth()->checkRole($roleName);
        if ($result instanceof Response) {
            throw new AuthException($result);
        }

        return $result;
    }

    /**
     * Fonction pour afficher un template
     *
     * @param string $tpl  name of the tpl to display
     * @param array  $data array of data to assign for use in the template
     * @param int $status
     *
     * @return mixed
     *
     * @throws \Exception
     * @throws \SmartyException
     */
    public function render($tpl, $data = array(), $status = 200)
    {
        if (!empty($data)) {
            $this->assign($data);
        }

        Application::getEventDispatcher()->dispatch(
            TemplateEvent::PRE_RENDER,
            new TemplateEvent($this->template)
        );

        // on enregistre passe les infos de session au template
        $this->template->assign('session', $this->request->getSession()->all());

        return $this->response($this->template->fetch($tpl), $status);
    }

    /**
     * permet de rendre un bout de template, sans le layout
     *
     * @param string $tpl  name of the tpl to display
     * @param array  $data array of data to assign for use in the template
     *
     * @return Response
     *
     * @throws \Exception
     * @throws \SmartyException
     */
    public function fetch($tpl, $data = array())
    {
        if (!empty($data)) {
            $this->assign($data);
        }

        return $this->response($this->template->fetch($tpl));
    }

    /**
     * Function pour générer un PDF via le contenu html
     *
     * @param string $htmlContent contenu HTML du PDF
     * @param string $fileName    nom du fichier a générer
     * @param string $header      type du pdf a rendre
     *
     * @return Response
     */
    public function renderPdf($htmlContent, $fileName, $header = 'Pdf_Legal')
    {
        if ($this->request->query->has('html')) {
            return $this->response($htmlContent);
        } else {
            $pdf = new Pdf;
            $pdf->generate($htmlContent, $fileName, $header);
        }
    }

    /**
     * Génère une réponse à renvoyer
     *
     * @param string $content contenu de la réponse
     * @param int    $status  code HTTP de la réponse
     * @param array  $headers headers complémentaire
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function response($content = '', $status = 200, $headers = array())
    {
        $objResponse = (in_array($this->request->getMimeType('json'), $this->request->getAcceptableContentTypes())) ?
            'Symfony\Component\HttpFoundation\JsonResponse' : 'Symfony\Component\HttpFoundation\Response';
        return new $objResponse($content, $status, $headers);
    }

    /**
     * Permet de récupérer la session de la requete courante
     *
     * @return \Symfony\Component\HttpFoundation\Session\SessionInterface
     */
    public function getSession()
    {
        return $this->request->getSession();
    }

    /**
     * Permet de récuperer le flashBag lié a la requete courante
     *
     * @return \Symfony\Component\HttpFoundation\Session\Flash\FlashBagInterface
     */
    public function getFlashBag()
    {
        return $this->getSession()->getFlashBag();
    }

    /**
     * Permet de faire une redirection vers une url
     *
     * @param string $url     l'url vers laquelle on veut rediriger l'internaute
     * @param int    $status  le code HTTP
     * @param array  $headers des headers complémentaire
     *
     * @return RedirectResponse
     */
    public function redirect($url, $status = 302, $headers = array())
    {
        return new RedirectResponse($url, $status, $headers);
    }

    /**
     * Retourne le type du device qui a lancé la requete : mobile / desktop
     *
     * @return string
     */
    public function getDeviceType()
    {
        return (Application::getMobileDetect()->isMobile() && !Application::getMobileDetect()->isTablet()) ?
            'mobile' : 'desktop';
    }

    /**
     * @deprecated
     *
     * @param array $indexes
     * @param array $optVars
     * @return array
     */
    public function getFlashAssign(array $indexes, array $optVars = array())
    {
        $flashBag = $this->getFlashBag();
        $vars = array();

        foreach ($indexes as $index) {
            if ($flashBag->has($index)) {
                $vars[$index] = $flashBag->get($index);
            }
        }

        if (!empty($optVars)) {
            $vars = array_merge($vars, $optVars);
        }

        return $vars;
    }

    /**
     * Return json response
     * Template can be given with its variables and assigned to json "html" index
     *
     * @deprecated
     *
     * @param array  $json
     * @param string $template
     * @param array  $vars
     *
     * @return mixed
     *
     * @throws \Exception
     * @throws \SmartyException
     */
    public function json(array $json, $template = "", array $vars = array())
    {
        $tpl = array();
        if (!empty($template)) {
            $this->template->assign($vars);
            $tpl["html"] = $this->template->fetch($template);
        }
        return $this->response(array_merge($json, $tpl));
    }
}
