<?php

namespace Shopbydedi;

use Event\PanierEvent;
use Shopbydedi\Shipping\Relay;

use Adresse;
use Livraison;
use Paiement;
use Client;
use Application;
use Produit;
use Attribut;
use Tools;
use DediException;
use Texte;
use Pays;
use Boutique;
use Promotion;
use Exception;
use Reduction;
use BonAchat;

/**
 * Class Panier
 * @package Shopbydedi
 */
class Panier
{
    static protected $instance = null;
    /**
     * @var array
     */
    protected $products;
    /**
     * @var Adresse
     */
    protected $addressShipping;
    /**
     * @var Adresse
     */
    protected $addressBilling;
    /**
     * @var Livraison
     */
    protected $shipping;
    /**
     * @var Paiement[]
     */
    protected $payments;
    /**
     * @var int
     */
    protected $reference;
    /**
     * @var Client
     */
    protected $customer;
    /**
     * @var \Shopbydedi\Shipping\Relay
     */
    protected $relay;
    /**
     * @var array
     */
    protected $reductions;
    /**
     * @var array
     */
    protected $bonAchats;
    /**
     * @var bool
     */
    protected $sameAddress;
    /**
     * @var string
     */
    protected $deviceType;
    /**
     * @var array
     */
    static protected $keyToGenerateIdentifier = array(
        'produit_id',
        'produit_attribut_id'
    );

    /**
     * Constructeur
     *
     */
    protected function __construct()
    {
        $this->init();
    }

    /**
     * Init le panier
     *
     * @return void
     */
    public function init()
    {
        $this->products = array();
        $this
            ->initAddresses()
            ->initRelay()
            ->initReduction()
            ->initBonAchats()
            ->initCustomer();
        $this->shipping = null;
        $this->payments = array();
        $this->reference = null;
        $this->sameAddress = true;

        $this->dispatch(PanierEvent::CHANGE);
    }

    /**
     * init paiement en supprimant tous les paiement avec un paiement_id
     *
     * @return $this
     */
    public function initPaiement()
    {
        if (!empty($this->payments)) {
            foreach ($this->payments as $k => $pay) {
                if (!empty($pay->paiement_id)) {
                    unset($this->payments[$k]);
                }
            }
        }

        if (empty($this->payments)) {
            $this->payments = array();
        }

        return $this;
    }

    /**
     * Function pour re init les adresses dans le panier
     *
     * @return $this
     */
    public function initAddresses()
    {
        $this->addressShipping = null;
        $this->addressBilling = null;
        return $this;
    }

    /**
     * @return $this
     */
    public function initCustomer()
    {
        $this->customer = null;
        return $this;
    }

    /**
     * @return $this
     */
    public function initRelay()
    {
        $this->relay = null;
        return $this;
    }

    /**
     * @return $this
     */
    public function initReduction()
    {
        $this->reductions = array();
        return $this;
    }

    /**
     * @return $this
     */
    public function initBonAchats()
    {
        $this->bonAchats = array();
        return $this;
    }

    /**
     * Charge le panier avec les infos en session
     *
     * @return Panier
     */
    public static function getInstance()
    {
        if (!(self::$instance instanceof self)) {
            $request = Application::getCurrentRequest();
            if (!empty($request)) {
                $session = $request->getSession();
                $cart = $session->get('cart');
            }
            if (!empty($cart)) {
                self::$instance = $cart;
            } else {
                self::$instance = new static();
            }
        }
        return self::$instance;
    }

    /**
     * @param Panier $panier
     */
    public static function set(Panier $panier)
    {
        self::$instance = $panier;
        self::$instance->dispatch(PanierEvent::CHANGE);
    }

    /**
     * Ajoute un produit au panier
     *
     * @param Produit       $produit  id produit a ajouté au panier
     * @param Attribut|null $attribut id attribut a ajouté au panier
     * @param int           $qte      quantité a ajouter au panier
     * @param array         $opts     tableau d'options suppl
     *
     * @throws DediException
     *
     * @return bool
     */
    public function addProduct(
        Produit $produit,
        Attribut $attribut = null,
        $qte = 1,
        $opts = array()
    ) {

        if ($qte <= 0) {
            throw new DediException('no quantity');
        }

        if (is_numeric($qte) == false) {
            $texte = new Texte();
            throw new DediException($texte->getOneOrCreate('errFormatQte', 'La quantité saisie n\'est pas correct.'));
        }

        if ($this->isAvailableToAdd($produit, $attribut, $qte, $opts) == false) {
            $texte = new Texte();
            throw new DediException(
                $texte->getOneOrCreate('errStockAppro', 'Ce produit n\'est pas disponible pour la quantité demandé')
            );
        }

        // on cherche le produit
        $pos = $this->find($produit, $attribut, $opts);

        if ($pos == -1) {

            $prodToCart = $produit->toArray();
            if (is_null($attribut) == false) {
                $prodToCart = array_merge($prodToCart, $attribut->toArray());
                $prodToCart['produit_prix_ht'] = $attribut->attribut_produit_prix_ht;
                $prodToCart['produit_prix_ttc'] = $attribut->attribut_produit_prix_ttc;
            }

            $prodToCart['quantite_order'] = $qte;
            $prodToCart['produit_total_ht'] = $prodToCart['produit_prix_ht'] * $qte;
            $prodToCart['produit_total_ttc'] = $prodToCart['produit_prix_ttc'] * $qte;
            $identifier = $this->generateIdentifier($prodToCart);
            $prodToCart['identifier'] = $identifier;
            $this->products[] = $prodToCart;

            // on lance un event pour dire que l'on va ajoute un produit au panier
            $this->dispatch(PanierEvent::ADD);

        } else {
            $this->updateProduct(
                $this->products[$pos]['identifier'],
                $this->products[$pos]['quantite_order'] + $qte
            );

            $identifier = $this->products[$pos]['identifier'];
        }

        $this->calculateShipping();

        $this->dispatch(PanierEvent::CHANGE);

        return $identifier;
    }

    /**
     * Met a jour la quantite d'un produit dans le panier
     *
     * @param string $identifier identifier du produit a mettre a jour
     * @param int    $qte        la nouvelle quantite du produit
     * @param float  $price      le prix du produit
     * @param float  $remise     valeur d'une remise eventuel
     *
     * @return bool
     * @throws DediException
     */
    public function updateProduct($identifier, $qte = null, $price = null, $remise = null)
    {
        $pos = $this->findByIdentifier($identifier);
        if ($pos == -1) {
            throw new DediException('no product with identifier ' . $identifier);
        }

        $produit = new Produit($this->products[$pos]['produit_id']);
        $attribut = (!empty($this->products[$pos]['produit_attribut_id'])) ?
            new Attribut($this->products[$pos]['produit_attribut_id']) : null;

        if (is_null($qte)) {
            $qte = $this->products[$pos]['quantite_order'];
        }

        if ($qte <= 0) {
            return $this->deleteProduct($identifier);
        }

        if ($this->isAvailableToAdd($produit, $attribut, $qte) == false) {
            $texte = new Texte();
            throw new DediException(
                $texte->getOneOrCreate('errStockAppro', 'Ce produit n\'est pas disponible pour la quantité demandée.')
            );
        }

        $this->products[$pos]['quantite_order'] = $qte;

        if (is_null($price) == false || is_null($remise) == false) {

            if (is_null($price)) {
                $price = round($this->products[$pos]['attribut_prix_ht'] - ($this->products[$pos]['attribut_prix_ht'] * ($remise / 100)), 2);
            }

            // prix origin pour calcul remise : attribut_prix_ht
            if (is_null($remise)) {
                $remise = round(($this->products[$pos]['attribut_prix_ht'] - $price) * 100 / $this->products[$pos]['attribut_prix_ht'], 2);
            }

            $this->products[$pos]['produit_prix_ht'] = $price;
            $this->products[$pos]['produit_prix_ttc'] = round($this->products[$pos]['produit_prix_ht'] * (1 + $this->products[$pos]['taxe_taux']), 2);

            $this->products[$pos]['produit_remise'] = $remise;
        }

        $this->products[$pos]['produit_total_ht'] = $this->products[$pos]['produit_prix_ht'] * $qte;
        $this->products[$pos]['produit_total_ttc'] = $this->products[$pos]['produit_prix_ttc'] * $qte;

        $this->calculateShipping();

        $this->dispatch(PanierEvent::UPDATE);
        $this->dispatch(PanierEvent::CHANGE);

        return true;

    }

    /**
     * Supprime un produit du panier
     *
     * @param string $identifier identifier du produit a supprimer
     *
     * @return bool
     * @throws DediException
     */
    public function deleteProduct($identifier)
    {
        $pos = $this->findByIdentifier($identifier);

        if ($pos == -1) {
            throw new DediException('no product identifier ' . $identifier);
        }

        unset($this->products[$pos]);

        $this->calculateShipping();

        $this->dispatch(PanierEvent::DELETE);
        $this->dispatch(PanierEvent::CHANGE);

        return true;
    }

    /**
     * check si le produit / attribut peut etre ajouté au panier
     *
     * @param Produit       $produit  id produit a ajouté au panier
     * @param Attribut|null $attribut id attribut a ajouté au panier
     * @param int           $qte      quantité a ajouter au panier
     * @param array         $opts     tableau d'options suppl
     *
     * @return bool
     */
    public function isAvailableToAdd(
        Produit $produit,
        Attribut $attribut = null,
        $qte = 1,
        $opts = array()
    ) {

        // @TODO : check type produit produit_type_stock
        // pour savoir si on fait une verif ou non

        if (is_null($attribut) == false) {
            $isAvailable = (
                $attribut->produit_attribut_quantite >= $qte
                || $attribut->produit_appro_id == 1
            );
        } else {
            $isAvailable = (
                $produit->prod_qte >= $qte
                || $produit->prod_appro == 1
            );
        }

        return $isAvailable;
    }

    public function calculateShipping($livraisonId = null)
    {
        $livraison = new Livraison();
        $pays = new Pays();

        //Si le nombre de produit du panier est nulle, on efface le shipping
        if ($this->getCountProducts() == 0) {
            $this->setShipping(null);
            return true;
        }

        list($totalHT, $totalTTC) = $this->getTotalProducts();

        $addr = $this->getAddressShipping();
        $paysId = (!empty($addr)) ?
            $addr->pays_id : $pays->get(['pays_iso_code_2' => Boutique::getInstance()->getLangue()->ini], 'pays_id');

        $arrLivraison = $livraison->get(array(
            'pays_id' => $paysId,
            'poids' => $this->getWeight(),
            'prix' => $totalHT,
            'livraison_type_id' => array(Livraison::TYPE_DOM, Livraison::TYPE_REL),
            'livraison_id' => $livraisonId
        ));

        if (!empty($arrLivraison)) {
            $paramLivr = array(
                'livraison_id' => $arrLivraison[0]['livraison_id'],
                'pays_id' => $paysId,
                'poids' => $this->getWeight(),
                'prix' => $totalHT,
            );
            if ($this->isTva()) {
                $paramLivr['prix_ttc'] = $totalTTC;
            } else {
                $paramLivr['prix_ht'] = $totalHT;
            }
            $arrPort = $livraison->get($paramLivr);

            if (!empty($arrPort)) {
                // les frais de port sont offerts ?
                if (isset($arrPort[0]['livraison_gratuit_reste']) && ($arrPort[0]['livraison_gratuit_reste'] <= 0)) {
                    $arrPort[0]['isgratuit'] = 1;
                } else {
                    $arrPort[0]['livraison_gratuit_reste'] = round($arrPort[0]['livraison_gratuit_reste']);
                }
                list($arrPort[0]['delai_min'], $arrPort[0]['delai_date_min']) = $this->getDelai('min', $arrPort[0]['livraison_delai_min']);
                list($arrPort[0]['delai_max'], $arrPort[0]['delai_date_max']) = $this->getDelai('max', $arrPort[0]['livraison_delai_max']);

                // si fdp gratos met les prix a 0
                $promotion = new Promotion();
                $promotion->validFDPGratuit($arrPort);

                $arrPort = $arrPort[0];

                $ship = new Livraison();
                $ship->hydrate($arrPort);
                $this->setShipping($ship);

                $this->dispatch(PanierEvent::CHANGE);
            }
        }
    }

    /**
     * Retourne la liste des produits du panier
     *
     * @return array
     */
    public function getProducts()
    {
        return $this->products;
    }

    /**
     * Retourne un produit
     *
     * @param int $pos
     *
     * @return array
     */
    public function getProductsByPos($pos)
    {
        return $this->products[$pos];
    }

    /**
     * Retourne un produit
     *
     * @param string $identifier
     *
     * @return array
     * @throws DediException
     */
    public function getProductsByIdentifier($identifier)
    {
        $pos = $this->findByIdentifier($identifier);

        if ($pos == -1) {
            throw new DediException('no product identifier ' . $identifier);
        }
        return $this->getProductsByPos($pos);
    }



    /**
     * Retourne le nombre de produits dans le panier
     *
     * @param bool $distinct si true : recupere le nombre de produits distinct
     * , sinon le nombre d'article dans le panier (produit * qte)
     *
     * @return int
     */
    public function getCountProducts($distinct = false)
    {
        if ($distinct == false) {
            $qte = 0;
            if (!empty($this->products)) {
                foreach ($this->products as $product) {
                    $qte+= $product['quantite_order'];
                }
            }

            return $qte;
        } else {
            return count($this->products);
        }
    }

    /**
     * retourne le poids d'un produit
     *
     * @return int
     */
    public function getWeight()
    {
        $weight = 0;
        if (!empty($this->products)) {
            foreach ($this->products as $product) {
                $weight+= $product['prod_poids'] * $product['quantite_order'];
            }
        }

        // spec Erika
        try {
            $panierConfig = Boutique::getInstance()->getConfig('panier');
            if (!empty($panierConfig['suppl_poids'])) {
                $weight += $panierConfig['suppl_poids'];
            }
        } catch (Exception $e) {

        }

        return $weight;
    }

    /**
     * Retourne le délai pour expedié le colis en jours
     *
     * @return int
     */
    public function getDelaiExpedie()
    {
        $delai = 1;
        // TODO : revoir niveau produit
        /*foreach ($this->products as $product) {
            if ($product['produit_attribut_quantite'] <= 0 && $product['produit_attribut_livraison'] > $delai) {
                $delai = $product['produit_attribut_livraison'];
            }
        }*/

        return $delai;
    }

    /**
     * Retourne le délai pour livrer les produits de la commande
     *
     * @param string $delai delai min / max pour la livraison
     * @param int    $delta delta de jour pour livré la commande
     *
     * @return array nombre de jours de delai et date de livraison
     */

    public function getDelai($delai, $delta = 0)
    {
        $nbDays = $this->getDelaiExpedie() + $delta;

        $prepare = false;
        while ($prepare == false) {
            if (Tools::dateCheckFree(mktime(0, 0, 0, date("m"), date("d") + $nbDays, date("Y"))) == 0) {
                $prepare = true;
            } else {
                $nbDays++;
            }
        }

        return array(
            $nbDays,
            mktime(0, 0, 0, date("m"), date("d") + $nbDays, date("Y"))
        );
    }

    /**
     * Cherche si un produit est déjà dans le panier,
     * si oui retourne sa position dans le tableau
     *
     * @param Produit       $produit  id du produit recherche
     * @param Attribut|null $attribut attribut id du produit recherché
     * @param array         $opts     tableau d'option,
     * le produit doit matché avec tous les champs du tableau
     * pour que l'on retourne sa position
     *
     * @return int
     */
    public function find(
        Produit $produit,
        Attribut $attribut = null,
        $opts = array()
    ) {

        $prodToCart = $produit->toArray();

        if (is_null($attribut) == false) {
            $prodToCart = array_merge($prodToCart, $attribut->toArray());
        }
        $prodToCart = array_merge($prodToCart, $opts);


        $identifier = $this->generateIdentifier($prodToCart);

        return $this->findByIdentifier($identifier);
    }

    /**
     * Cherche un produit dans le panier avec son identifier
     *
     * @param string $identifier chaine unique du produit dans le panier
     *
     * @return integer
     */
    public function findByIdentifier($identifier)
    {
        if (!empty($this->products)) {
            foreach ($this->products as $pos => $product) {
                if ($product['identifier'] == $identifier) {
                    return $pos;
                }
            }
        }

        return -1;
    }

    /**
     * Fonction pour savoir si un produit dans le panier a la propriete et valeur
     *
     * @param string $property nom de l'index du tableau
     * @param string $value    valeur de l'index du tableau pour le match
     *
     * @return bool
     */
    public function have($property, $value)
    {
        $have = false;

        if (!empty($this->products)) {
            foreach ($this->products as $product) {
                if (isset($product[$property]) && $product[$property] == $value) {
                    $have = true;
                    break;
                }
            }

        }

        return $have;
    }

    /**
     * Retourne un identifiant unique pour chaque produit ajouter au panier
     *
     * @param array $prodToCart infos du produit dans le panier
     *
     * @return string
     * @throws DediException
     */
    public function generateIdentifier($prodToCart)
    {
        $identifier = '';
        foreach (self::$keyToGenerateIdentifier as $ktgi) {
            if (!empty($prodToCart[$ktgi])) {
                $identifier.= $prodToCart[$ktgi];
            }
        }

        if (empty($identifier)) {
            throw new DediException(
                'manque infos pour générer identifier produit pour le panier'
            );
        }

        return md5($identifier);
    }

    /**
     * Init adresse de livraison
     *
     * @param Adresse $address objet qui represente l'adresse de livraison
     *
     * @return $this
     */
    public function setAddressShipping(Adresse $address)
    {
        $this->addressShipping = $address;

        $this->dispatch(PanierEvent::CHANGE);

        return $this;
    }

    /**
     * retourne adresse de livraison
     *
     * @return Adresse
     */
    public function getAddressShipping()
    {
        return $this->addressShipping;
    }

    /**
     * check si adresse de livraison ok, si on a toutes les infos
     *
     * @return bool
     */
    public function isValidAddressShipping()
    {
        return $this->isValidAddress($this->addressShipping);
    }

    /**
     * Init adresse de facturation
     *
     * @param Adresse $address objet qui represente l'adresse de facturation
     *
     * @return $this
     */
    public function setAddressBilling(Adresse $address)
    {
        $this->addressBilling = $address;

        $this->dispatch(PanierEvent::CHANGE);

        return $this;
    }

    /**
     * get adresse de facturation
     *
     * @return Adresse
     */
    public function getAddressBilling()
    {
        return $this->addressBilling;
    }

    /**
     * check si adresse de facturation ok, si on a toutes les infos
     *
     * @return bool
     */
    public function isValidAddressBilling()
    {
        return $this->isValidAddress($this->addressBilling);
    }

    /**
     * Check si une adresse est bonne
     *
     * @param Adresse $address objet adresse a verifier
     *
     * @return bool
     */
    public function isValidAddress(Adresse $address)
    {
        return (!empty($address->civilite_nom_nom)
            && !empty($address->adresse_nom)
            && !empty($address->adresse_prenom)
            && !empty($address->adresse_rue)
            && !empty($address->adresse_ville)
            && !empty($address->adresse_cp)
            && !empty($address->pays_id)
        );
    }

    /**
     * @param Relay $relay
     * @return $this
     */
    public function setRelay(Relay $relay)
    {
        $this->relay = $relay;

        $this->dispatch(PanierEvent::CHANGE);

        return $this;
    }

    /**
     * @return Relay
     */
    public function getRelay()
    {
        return $this->relay;
    }

    /**
     * Savoir si le panier est soumis a la TVA ou non
     *
     * @return bool
     */
    public function isTva()
    {
        if (!empty($this->addressShipping->adresse_id)) {
            return (new Pays($this->addressShipping->pays_id))->isTva();
        } else {
            return true;
        }
    }

    /**
     * Init le mode de livraison
     *
     * @param Livraison $shipping objet qui represente la livraison choisie
     *
     * @return $this
     */
    public function setShipping($shipping)
    {

        $this->shipping = $shipping;

        $this->dispatch(PanierEvent::CHANGE);

        return $this;
    }

    /**
     * Retourne le mode de livraison
     *
     * @return Livraison
     */
    public function getShipping()
    {
        return $this->shipping;
    }

    /**
     * associe un client au panier
     *
     * @param Client $client objet client
     *
     * @return Panier
     */
    public function setCustomer(Client $client)
    {
        $this->customer = $client;
        $this->dispatch(PanierEvent::CHANGE);
        return $this;
    }

    /**
     * retourne le client associé au panier
     *
     * @return Client
     */
    public function getCustomer()
    {
        return $this->customer;
    }


    /**
     * @param $reference
     *
     * @return $this
     */
    public function setReference($reference)
    {
        $this->reference = $reference;
        $this->dispatch(PanierEvent::CHANGE);
        return $this;
    }

    /**
     * @return string
     */
    public function getDeviceType()
    {
        return $this->deviceType;
    }

    /**
     * @param $deviceType
     *
     * @return $this
     */
    public function setDeviceType($deviceType)
    {
        $this->deviceType = $deviceType;
        $this->dispatch(PanierEvent::CHANGE);
        return $this;
    }

    /**
     * @return int
     */
    public function getReference()
    {
        return $this->reference;
    }

    /**
     * @param $sameAddress
     *
     * @return $this
     */
    public function setSameAddress($sameAddress)
    {
        $this->sameAddress = $sameAddress;
        $this->dispatch(PanierEvent::CHANGE);
        return $this;
    }

    /**
     * @return bool
     */
    public function getSameAddress()
    {
        return $this->sameAddress;
    }

    /**
     * Ajoute un mode de paiement
     *
     * @param float    $paie    montant du paiement
     * @param Paiement $payment objet paiement
     *
     * @return Panier
     */
    public function addPayment($paie, Paiement $payment)
    {
        $payment->paiement_montant = $paie;
        $this->payments[] = $payment;

        $this->dispatch(PanierEvent::CHANGE);

        return $this;
    }

    /**
     * Retourne la liste des paiements du panier
     *
     * @return Paiement[]
     */
    public function getPayment()
    {
        return $this->payments;
    }

    /**
     * Ajoute un code promo
     *
     * @param Reduction $reduction objet reduction qui represente un code promo
     *
     * @return Panier
     */
    public function addReduction(Reduction $reduction)
    {
        $this->reductions[] = $reduction;

        $this->dispatch(PanierEvent::CHANGE);

        return $this;
    }

    /**
     * Retourne la liste des codes de reduction du panier
     *
     * @return \Reduction[]
     */
    public function getReduction()
    {
        return $this->reductions;
    }

    /**
     * cherche la reduction passé en paramètre
     *
     * @param \Reduction $reduction
     *
     * @return bool
     */
    public function haveReduction($reduction)
    {
        if (!empty($this->reductions)) {
            foreach ($this->reductions as $reduc) {
                if ($reduc->code == $reduction->code) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Return true if the promotion type given can be added to the cart, false otherwise
     *
     * @param $cumulableId
     */
    public function canApplyType($cumulableId)// '1'
    {
        $cumulableId = ($cumulableId == Promotion::PLURALITY_OK ? Promotion::PLURALITY_OK : Promotion::PLURALITY_NOT_OK);
        $return = false;

        // first promotion, no need for tests
        if (empty($this->reductions)) {
            $return = true;
        }

        switch ($cumulableId) {
            // sum of many
            case Promotion::PLURALITY_OK:
                $return = true; // OK except if another type is already there
                foreach ($this->reductions as $reduc) {
                    if ($reduc->cumul != $cumulableId) {
                        $return = false; // one of the promotion is different, we cannot add another
                        break;
                    }
                }
                break;

            // only one
            case Promotion::PLURALITY_NOT_OK:
                // nothing to do, can only be applied if first reduction
                break;

            // type not matched
            default:
                // shouldn't happen, default to false
                $return = false;
                break;
        }

        return $return;
    }

    public function refreshReduction()
    {
        // TODO : raffraichit le montant des réductions
    }

    /**
     * Ajoute un bon d'achat
     *
     * @param BonAchat $ba objet bon d'achat qui represente un ba client
     *
     * @return Panier
     */
    public function addBonAchat(BonAchat $ba)
    {
        $this->bonAchats[] = $ba;

        $this->dispatch(PanierEvent::CHANGE);

        return $this;
    }

    /**
     * @param BonAchat $ba
     *
     * @return bool
     *
     * @throws DediException
     */
    public function deleteBonAchat(BonAchat $ba)
    {
        $pos = $this->findBonAchat($ba);
        if ($pos === false) {
            throw new DediException("This BA doesn't exist");
        }

        unset($this->bonAchats[$pos]);
        $this->dispatch(PanierEvent::CHANGE);
        return true;
    }

    /**
     * @param BonAchat $ba
     *
     * @return bool|int
     */
    public function findBonAchat(BonAchat $ba)
    {
        $pos = false;
        if (!empty($this->bonAchats)) {
            foreach ($this->bonAchats as $k => $bonAchat) {
                if ($ba->bon_achat_id == $bonAchat->bon_achat_id) {
                    $pos = $k;
                    break;
                }
            }
        }

        return $pos;
    }

    /**
     * Retourne la liste des bon d'achats du panier
     *
     * @return BonAchat[]
     */
    public function getBonAchat()
    {
        return $this->bonAchats;
    }

    /**
     * return un array avec le total HT et total TTC des produits du panier
     * auquel on enleve les codes promos utilisés
     *
     * @param $reductions bool
     *
     * @return array
     */
    public function getTotalProducts($reductions = true)
    {
        $totalHT = $totalTTC = 0;
        if (!empty($this->products)) {
            foreach ($this->products as $product) {
                $totalHT += $product['attribut_prix_ht'] * $product['quantite_order'];
                $totalTTC += $product['attribut_prix_ttc'] * $product['quantite_order'];
            }
        }

        // code promos
        $totalReductionHT = $totalReductionTTC = 0;
        if (!empty($this->reductions) && $reductions) {
            foreach ($this->reductions as $reduction) {
                $totalReductionHT += $reduction->price - $reduction->taxe;
                $totalReductionTTC += $reduction->price;
            }
        }

        return array($totalHT - $totalReductionHT, $totalTTC - $totalReductionTTC);
    }

    /**
     * return un array avec le montant des produits + frais de port HT et TTC
     *
     * @return array
     */
    public function getTotalProductsAndShip()
    {
        list($totalHT, $totalTTC) = $this->getTotalProducts();

        if (!empty($this->shipping)) {
            $totalHT+= $this->shipping->port_ht;
            $totalTTC+= $this->shipping->port_ttc;
        }

        return array($totalHT, $totalTTC);
    }

    /**
     * return un array avec le montant HT et TTC
     * a payer : total des produits + frais de livraison - deduction des remises
     *
     * @return float
     */
    public function getAPayer()
    {
        list($totalHT, $totalTTC) = $this->getTotalProductsAndShip();

        $totalBa = 0;
        if (!empty($this->bonAchats)) {
            foreach ($this->bonAchats as $bonAchat) {
                $totalBa+= $bonAchat->bon_achat_prix;
            }
        }

        // TODO : voir comment faire pour le HT
        $aPayerHT = $totalHT;
        $aPayerTTC = $totalTTC - $totalBa;

        return array($aPayerHT, $aPayerTTC);
    }

    /**
     * fonction appelé lorsqu'on appelle serialize sur l'objet
     *
     * @return array
     */
    public function __sleep()
    {
        $array = get_object_vars($this);

        $vars = array();
        foreach ($array as $k => $v) {
            array_push($vars, $k);
        }

        return $vars;
    }

    /**
     * @param $eventName
     */
    protected function dispatch($eventName)
    {
        Application::getEventDispatcher()->dispatch($eventName, new PanierEvent($this));
    }
}
